/*
	Fractal - A program to calculate Newton fractals.
	Copyright (C) 2004 Johannes Bauer
	
	This file is part of Fractal.

    Fractal is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Fractal is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Fractal; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Johannes Bauer
	Mussinanstr. 140
	92318 Neumarkt/Opf.
	JohannesBauer@gmx.de
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define MODE_NORMAL 		0
#define MODE_INVERTED 		1
#define EPSILON 			(double)0.00000000001
#define EPSILON_INVERTED	(double)3.00000000000

double absolute(double x) {
	if (x<0) return -x;
	return x;
}

int main(int argc, char **argv) {
	double R1, I1, R2, I2, Scale;
	int Cnt, i;
	double RLen, RMid, ILen, IMid;
	char Mode;
	
	if (argc!=8) {
		fprintf(stderr,"Parameter!\n");
		return 1;
	}

	if (strcmp(argv[1],"normal")==0) {
		Mode=MODE_NORMAL;
	} else {
		if (strcmp(argv[1],"inverted")==0) {
			Mode=MODE_INVERTED;
		} else {
			fprintf(stderr,"First parameter must either be 'normal' or 'inverted'.\n");
			return 1;
		}
	}
	
	R1=atof(argv[2]);
	I1=atof(argv[3]);
	R2=atof(argv[4]);
	I2=atof(argv[5]);
	Scale=atof(argv[6]);
	Cnt=atoi(argv[7]);

	if (Mode==MODE_INVERTED) {
		// Now zoom out until EPSILON_INVERTED is reached
		fprintf(stderr,"Calculation inverted position...\n");
		i=0;
		while (i<Cnt) {
			RLen=R2-R1;
			RMid=(R1+R2)/(double)2.0;
			RLen/=Scale;
			R1=RMid-(RLen/(double)2.0);
			R2=RMid+(RLen/(double)2.0);
		
			ILen=I2-I1;
			IMid=(I1+I2)/(double)2.0;
			ILen/=Scale;
			I1=IMid-(ILen/(double)2.0);
			I2=IMid+(ILen/(double)2.0);

			if ((absolute(R1-R2)>EPSILON_INVERTED) ||
			    (absolute(I1-I2)>EPSILON_INVERTED)) {
				break;
			}
			i++;
		}
		fprintf(stderr,"Calculation done after %d iterations.\n",i);
		fprintf(stderr,"Results: R1 = %.46f\n",R1);
		fprintf(stderr,"         I1 = %.46f\n",I1);
		fprintf(stderr,"         R2 = %.46f\n",R2);
		fprintf(stderr,"         I2 = %.46f\n",I2);
	}
	
	for (i=1;i<=Cnt;i++) {
		RLen=R2-R1;
		RMid=(R1+R2)/(double)2.0;
		RLen*=Scale;
		R1=RMid-(RLen/(double)2.0);
		R2=RMid+(RLen/(double)2.0);
		
		ILen=I2-I1;
		IMid=(I1+I2)/(double)2.0;
		ILen*=Scale;
		I1=IMid-(ILen/(double)2.0);
		I2=IMid+(ILen/(double)2.0);

		if ((absolute(R1-R2)<EPSILON) ||
		    (absolute(I1-I2)<EPSILON)) {
			break;
		}
		
		printf("%.46f %.46f %.46f %.46f Picture_%04d.png\n",R1,I1,R2,I2,i);
	}
	
	return 0;
}
